//
//  Kmpg
// 
//  Copyright (C) 1997 Bjarni Ivarsson
//  email: Bjarni.Ivarsson@swipnet.se
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <qpainter.h>
#include <qtooltip.h>

#include "trackcalendar.h"

TrackCalendarArrow::TrackCalendarArrow( QWidget *parent, Direction dir ):
   QWidget( parent )
{
   _direction = dir;
   setBackgroundColor( QColor( 0, 0, 0 ) );
   if( dir == Up )
      QToolTip::add( this, "Up" );
   else
      QToolTip::add( this, "Down" );
   setFixedSize( 15, 11 );
}

void TrackCalendarArrow::paintEvent( QPaintEvent* )
{
   QPainter painter;
   
   painter.begin( this );
   painter.setPen( QColor( 255, 255, 0 ) );
   painter.drawRect( 0, 0, width(), height() );
   painter.end();
}

void TrackCalendarArrow::mousePressEvent( QMouseEvent *e )
{
   if( e->button() == LeftButton )
      emit( clicked() );
}

TrackCalendarItem::TrackCalendarItem( QWidget *parent, int index, KURL url ):
   QWidget( parent ), Led()
{
   _index = index;
   _url = url;
   _played = FALSE;
   adjustSize();
   setBackgroundColor( QColor( 0, 0, 0 ) );
   QToolTip::add( this, url.path() );
}

void TrackCalendarItem::adjustSize()
{
   if( _index < 99 )
      setFixedSize( 13, 11 );
   else if( _index < 199 )
      setFixedSize( 15, 11 );
   else
      setFixedSize( 18, 11 );
}

void TrackCalendarItem::paintEvent( QPaintEvent* )
{
   QPainter painter;
   
   painter.begin( this );
   painter.setPen( QColor( 0, (_played?127:255), 0 ) );
   painter.drawRect( 0, 0, width(), height() );
   if( _index < 99 )
   {
      if( _index >= 9 )
	 drawDigit( &painter, 2, 2, 3, ((_index+1)/10)%10 );
      drawDigit( &painter, 7, 2, 3, (_index+1)%10 );
   }
   else if( _index < 199 )
   {
      drawDigit( &painter, -1, 2, 3, 1 );
      drawDigit( &painter, 4, 2, 3, ((_index+1)/10)%10 );
      drawDigit( &painter, 9, 2, 3, (_index+1)%10 );
   }
   else
   {
      drawDigit( &painter, 2, 2, 3, ((_index+1)/100)%100 );
      drawDigit( &painter, 7, 2, 3, ((_index+1)/10)%10 );
      drawDigit( &painter, 12, 2, 3, (_index+1)%10 );
   }
   painter.end();
}

void TrackCalendarItem::mousePressEvent( QMouseEvent *e )
{
   if( e->button() == LeftButton )
      emit( sigSelected( _index ) );
}

TrackCalendar::TrackCalendar( QWidget *parent, PlaylistMgr* mgr ):
   QWidget( parent )
{
   _list.setAutoDelete( TRUE );
   _mgr = mgr;
   _rows = -1;
   _toprow = 1;

   _panel = new QWidget( this );
   _panel->move( 0, 0 );
   _panel->setBackgroundColor( QColor( 0, 0, 0 ) );
   setBackgroundColor( QColor( 0, 0, 0 ) );

   _arrowUp = new TrackCalendarArrow( this, TrackCalendarArrow::Up );
   _arrowUp->move( 185, 1 );
   _arrowUp->hide();
   _arrowDown = new TrackCalendarArrow( this, TrackCalendarArrow::Down );
   _arrowDown->move( 185, 57 );
   _arrowDown->hide();
   connect( _arrowUp, SIGNAL(clicked()), SLOT(arrowUp()) );
   connect( _arrowDown, SIGNAL(clicked()), SLOT(arrowDown()) );

   resizeCalendar();

   connect( mgr, SIGNAL(sigClear()), 
	    SLOT(doClear()) );
   connect( mgr, SIGNAL(sigInsert( int, QString )),
	    SLOT(doInsert( int, QString )) );
   connect( mgr, SIGNAL(sigUpdate( int, QString )),
	    SLOT(doUpdate( int, QString )) );
   connect( mgr, SIGNAL(sigRemove( int )),
	    SLOT(doRemove( int )) );
   connect( mgr, SIGNAL(sigMoveUp( int )),
	    SLOT(doMoveUp( int )) );
   connect( mgr, SIGNAL(sigMoveDown( int )),
	    SLOT(doMoveDown( int )) );
   connect( mgr, SIGNAL(sigSetPlayed( int, bool )),
	    SLOT(doSetPlayed( int, bool )) );
   connect( mgr, SIGNAL(sigSetAllPlayed( bool )),
	    SLOT(doSetAllPlayed( bool )) );
   connect( mgr, SIGNAL(sigTrackSelected( int )),
	    SLOT(doTrackSelected( int )) );
}

TrackCalendar::~TrackCalendar()
{
}

void TrackCalendar::arrowUp()
{
   if( _toprow == 0 )
      return;
   _toprow--;
   _panel->move( 0, -_toprow*14 );
   if( _toprow == 0 )
      _arrowUp->hide();
   _arrowDown->show();
}

void TrackCalendar::arrowDown()
{
   if( _toprow >= ( _rows-5 ) )
      return;
   _toprow++;
   _panel->move( 0, -_toprow*14 );   
   if( _toprow >= ( _rows-5 ) )
      _arrowDown->hide();
   _arrowUp->show();
}

void TrackCalendar::doClear()
{
   _list.clear();
   resizeCalendar();
}

void TrackCalendar::doInsert( int index, QString str )
{
   TrackCalendarItem *item = new TrackCalendarItem( _panel, index, KURL(str) );

   _list.insert( index, item );
   connect( item, SIGNAL(sigSelected(int)), _mgr, SLOT(playTrack(int)) );
   positionTrack( index );
   resizeCalendar();
}

void TrackCalendar::doUpdate( int index, QString str)
{
}

void TrackCalendar::doRemove( int index )
{
   TrackCalendarItem *item;
   int i;

   _list.remove( index );
   if( _list.count() == index )
      return; // we just removed the last item
   for( item = _list.at( index ), i = index; 
	item != 0; 
	item = _list.next(), i++ )
   {
      item->setIndex( i );
      positionTrack( i );
   }
   
   resizeCalendar();
}

void TrackCalendar::doMoveUp( int index )
{
}

void TrackCalendar::doMoveDown( int index )
{
}

void TrackCalendar::doSetPlayed( int index, bool pl )
{
   _list.at( index )->setPlayed( pl );
}

void TrackCalendar::doSetAllPlayed( bool pl )
{
   TrackCalendarItem *item;
   
   for( item = _list.first(); item != 0; item = _list.next() )
      item->setPlayed( pl );
}

void TrackCalendar::doTrackSelected( int index )
{
}

void TrackCalendar::positionTrack( int index )
{
   TrackCalendarItem *item;
   int row, column;

   item = _list.at( index );

   if( index < 99 )
   {
      row = index/14;
      column = index%14;
      item->move( 3+column*14, 1+row*14 );
   }
   else if( index < 110 )
   {
      row = 7;
      column = index - 99;
      item->move( 17+column*16, 1+row*14 );
   }
   else if( index < 199 )
   {
      row = 8 + (index-110)/12;
      column = (index - 110)%12;
      item->move( 3+column*16, 1+row*14 );
   }
   else if( index < 205 )
   {
      row = 15;
      column = index - 199;
      item->move( 83+column*19, 1+row*14 );
   }
   else
   {
      row = 16 + (index-205)/10;
      column = (index-205)%10;
      item->move( 3+column*19, 1+row*14 );
   }
   item->show();
}

void TrackCalendar::resizeCalendar()
{
   int rows;
   
   if( _list.count() < 111 )
      rows = 1 + (_list.count()-1)/14;
   else if( _list.count() < 206 )
      rows = 9 + (_list.count()-111)/12;
   else
      rows = 17 + (_list.count()-206)/10;
   if( rows == _rows )
      return;
   _rows = rows;
   _panel->setFixedSize( 202, 2+rows*14 );
   if( rows>5 )
      rows = 5;
   if( _list.count()>0 )
      setFixedSize( 202, (rows?(2+rows*14):0) );
   else
      setFixedSize( 202, 1 );

   if( _rows <= 5 )
   {
      _toprow = 0;
      _panel->move( 0, 0 );
   }
   if( _toprow == 0 )
      _arrowUp->hide();
   if( _toprow >= ( _rows-5 ) )
      _arrowDown->hide();
   else
      _arrowDown->show();

   emit( sigSizeChanged() );
}

   

